// src/SubProcess.hh
// (C) 2006 - 2017 Philip Endecott
// This file is part of Anyterm; see http://anyterm.org/

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef SubProcess_hh
#define SubProcess_hh

#include <string>
#include <memory>
#include <functional>
#include <thread>
#include <atomic>

#include "FileDescriptor.hh"


class SubProcess_base
{
  // This base class is so that we can sensibly initialise pid and fd.
protected:
  int fd_num;
  ::pid_t pid;

public:
  SubProcess_base(std::string command, int pty_rows=25, int pty_cols=80);

};


class SubProcess: private SubProcess_base
{
  pbe::FileDescriptor fd;
  std::atomic<bool> terminating;

public:
  typedef std::function<void(std::string)> onOutput_t;
  typedef std::function<void(std::string)> onError_t;

  SubProcess(onOutput_t onOutput_,
             onError_t onError_,
	     std::string command,
	     int pty_rows=25, int pty_cols=80);

  ~SubProcess();

  void send(std::string s);
  void resize(int rows, int cols);

private:
  const onOutput_t onOutput;
  const onError_t onError;
  std::unique_ptr<std::thread> output_processor;
  void process_output(void);
};


#endif


